@@ -13,7 +13,7 @@
  *      Anatoly Vorobey <mellon@pobox.com>
  *      Brad Fitzpatrick <brad@danga.com>
  *
- *  $Id: memcached.c,v 1.42 2003/10/08 19:25:37 bradfitz Exp $
+ *  $Id: memcached.c,v 1.44 2003/12/01 21:18:22 avva Exp $
  */
 
 #include "config.h"
@@ -91,6 +91,7 @@ void settings_init(void) {
     settings.maxbytes = 64*1024*1024; /* default is 64MB */
     settings.maxconns = 1024;         /* to limit connections-related memory to about 5MB */
     settings.verbose = 0;
+    settings.oldest_live = 0;
 }
 
 conn **freeconns;
@@ -604,6 +605,10 @@ void process_command(conn *c, char *command) {
             if (it && (it->it_flags & ITEM_DELETED)) {
                 it = 0;
             }
+            if (settings.oldest_live && it && 
+                it->time <= settings.oldest_live) {
+                it = 0;
+            }
             if (it && it->exptime && it->exptime < now) {
                 item_unlink(it);
                 it = 0;
@@ -667,6 +672,12 @@ void process_command(conn *c, char *command) {
         return;
     }
 
+    if (strcmp(command, "flush_all") == 0) {
+        settings.oldest_live = time(0);
+        out_string(c, "OK");
+        return;
+    }
+
     if (strcmp(command, "version") == 0) {
         out_string(c, "VERSION " VERSION);
         return;
@@ -1305,21 +1316,18 @@ int main (int argc, char **argv) {
         }
     }
 
-    /* initialize other stuff */
-    item_init();
-    event_init();
-    stats_init();
-    assoc_init();
-    conn_init();
-    slabs_init(settings.maxbytes);
+    /* 
+     * initialization order: first create the listening socket
+     * (may need root on low ports), then drop root if needed,
+     * then daemonise if needed, then init libevent (in some cases
+     * descriptors created by libevent wouldn't survive forking).
+     */
 
-    /* lock paged memory if needed */
-    if (lock_memory) {
-#ifdef HAVE_MLOCKALL
-        mlockall(MCL_CURRENT | MCL_FUTURE);
-#else
-        fprintf(stderr, "warning: mlockall() not supported on this platform.  proceeding without.\n");
-#endif
+    /* create the listening socket and bind it */
+    l_socket = server_socket(settings.port);
+    if (l_socket == -1) {
+        fprintf(stderr, "failed to listen\n");
+        exit(1);
     }
 
     /* lose root privileges if we have them */
@@ -1338,6 +1346,7 @@ int main (int argc, char **argv) {
         }
     }
 
+    /* daemonize if requested */
     if (daemonize) {
         int res;
         res = daemon(0, settings.verbose);
@@ -1347,6 +1356,24 @@ int main (int argc, char **argv) {
         }
     }
 
+
+    /* initialize other stuff */
+    item_init();
+    event_init();
+    stats_init();
+    assoc_init();
+    conn_init();
+    slabs_init(settings.maxbytes);
+
+    /* lock paged memory if needed */
+    if (lock_memory) {
+#ifdef HAVE_MLOCKALL
+        mlockall(MCL_CURRENT | MCL_FUTURE);
+#else
+        fprintf(stderr, "warning: mlockall() not supported on this platform.  proceeding without.\n");
+#endif
+    }
+
     /*
      * ignore SIGPIPE signals; we can use errno==EPIPE if we
      * need that information
@@ -1359,13 +1386,6 @@ int main (int argc, char **argv) {
         exit(1); 
     }
 
-    /* create the listening socket and bind it */
-    l_socket = server_socket(settings.port);
-    if (l_socket == -1) {
-        fprintf(stderr, "failed to listen\n");
-        exit(1);
-    }
-
     /* create the initial listening connection */
     if (!(l_conn = conn_new(l_socket, conn_listening, EV_READ | EV_PERSIST))) {
         fprintf(stderr, "failed to create listening connection");
